package eu.ztsh.lfr.core.impl;

import eu.ztsh.lfr.core.TemperaturesService;
import eu.ztsh.lfr.model.Average;
import eu.ztsh.lfr.model.Averages;
import eu.ztsh.lfr.model.events.FileProcessedEvent;
import jakarta.annotation.Nonnull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

@Service
public class TemperaturesServiceImpl implements TemperaturesService {

    private static final Logger log = LoggerFactory.getLogger(TemperaturesServiceImpl.class);
    private final AtomicReference<Averages> averages = new AtomicReference<>();

    @Nonnull
    @Override
    public synchronized List<Average> getTemperaturesFor(String city) {
        while (averages.get() == null) {
            try {
                wait();
            } catch (InterruptedException e) {
                log.error("Thread was interrupted", e);
                Thread.currentThread().interrupt();
            }
        }
        return averages.get().get(city);
    }

    @EventListener(FileProcessedEvent.class)
    public synchronized void updateTemperatures(FileProcessedEvent event) {
        averages.set(event.averages());
        notifyAll();
    }

}
